Package: artma
Title: Automatic Replication Tools for Meta-Analysis
Version: 0.3.3
Authors@R: 
    person("Petr", "Čala", , "61505008@fsv.cuni.cz", role = c("aut", "cre"))
Author: Petr Čala [aut, cre]
Maintainer: Petr Čala <61505008@fsv.cuni.cz>
Description: Provides a unified and straightforward interface for
    performing a variety of meta-analysis methods directly from user data.
    Users can input a data frame, specify key parameters, and effortlessly
    execute and compare multiple common meta-analytic models. Designed for
    immediate usability, the package facilitates transparent, reproducible
    research without manual implementation of each analytical method.
    Ideal for researchers aiming for efficiency and reproducibility, it
    streamlines workflows from data preparation to results interpretation.
License: GPL-3
URL: https://github.com/PetrCala/artma
BugReports: https://github.com/PetrCala/artma/issues
Depends: R (>= 4.0.0)
Imports: cli (>= 3.6.5), climenu (>= 0.1.3), ggplot2 (>= 3.4.0), ggtext
        (>= 0.1.2), lintr (>= 3.2.0), lmtest (>= 0.9-40), memoise (>=
        2.0.1), metafor (>= 4.8-0), NlcOptim (>= 0.6), plm (>= 2.6-3),
        Rcpp (>= 1.0.12), rlang (>= 1.1.6), sandwich (>= 3.1-0), withr
        (>= 3.0.2), yaml (>= 2.3.10)
LinkingTo: Rcpp
Suggests: AER (>= 1.2-10), BMS (>= 0.3.4), box (>= 1.2.0), box.linters
        (>= 0.10.6), covr (>= 3.6.4), devtools (>= 2.4.5), fdrtool (>=
        1.2.17), fs (>= 1.6.6), here (>= 1.0.2), ivmodel (>= 1.9.0),
        knitr (>= 1.50), languageserver (>= 0.3.16), MAIVE (>= 0.1.10),
        mathjaxr (>= 1.8-0), mice (>= 3.16.0), optparse (>= 1.7.5),
        pkgbuild (>= 1.4.8), quadprog (>= 1.5-8), rddensity (>= 2.5),
        remotes (>= 2.5.0), rex (>= 1.2.1), rmarkdown (>= 2.30),
        roxygen2 (>= 7.3.3), testthat (>= 3.2.3)
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/testthat/parallel: TRUE
Config/testthat/start-first: github-actions, release
Encoding: UTF-8
RoxygenNote: 7.3.3
SystemRequirements: JAGS >= 4.3.1 (https://mcmc-jags.sourceforge.io/)
NeedsCompilation: yes
Packaged: 2026-02-11 12:38:16 UTC; runner
Repository: CRAN
Date/Publication: 2026-02-11 13:20:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 05:40:08 UTC; unix
Archs: artma.so.dSYM
