# Generated by roxygen2: do not edit by hand

S3method(print,artma_box_plot)
S3method(print,artma_funnel_plot)
S3method(print,artma_t_stat_histogram)
export(artma)
export(autonomy.describe)
export(autonomy.get)
export(autonomy.is_full)
export(autonomy.is_set)
export(autonomy.levels)
export(autonomy.set)
export(config.fix)
export(config.get)
export(config.overrides)
export(config.reset)
export(config.set)
export(data.preview)
export(methods.list)
export(options.copy)
export(options.create)
export(options.delete)
export(options.fix)
export(options.help)
export(options.list)
export(options.load)
export(options.modify)
export(options.open)
export(options.print_default_dir)
export(options.remove)
export(options.validate)
export(results.dir)
export(results.open)
export(viz.get)
export(viz.set)
export(viz.themes)
importFrom(Rcpp,evalCpp)
importFrom(lintr,Linter)
importFrom(lintr,indentation_linter)
importFrom(lintr,make_linter_from_xpath)
useDynLib(artma, .registration = TRUE)
